%normalgui
% Graphical User Interface for calculating probabilities associated with
% Skew-Normal distribution problems, and also the generation of samples and
% frequency tables which derive from a specific skew-normal distribution.
%
% LAST UPDATED by Andy French July 2014

function varargout = normalgui(varargin)

% Begin GUI initialization code
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @normalgui_OpeningFcn, ...
    'gui_OutputFcn',  @normalgui_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

%%

% GUI opening function
function normalgui_OpeningFcn(hObject, eventdata, handles, varargin)

%Define gloabl structures h and d. Structure h stores the GUI handles, d stores the GUI data
global h d
h = handles;

%Use defaults to initialize GUI data
d = normalgui_defaults;

%Update GUI using parameters stored in structure d
update_gui_edit_boxes_from_d;
update_pdf_graph;
generate_new_samples;
update_histogram;

%%

% GUI output function
function varargout = normalgui_OutputFcn(hObject, eventdata, handles)

%%

% Distribution mean edit box
function EDITmean_Callback(hObject, eventdata, handles)
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

% Distribution standard deviation edit box
function EDITstd_Callback(hObject, eventdata, handles)
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

% Edit box defining the distribution skew parameter
% Note absolute value must be < 1.
function EDITskew_Callback(hObject, eventdata, handles)
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

% Lower x value for probability calculation P( x_lower <= x < x_upper )
function EDITlowerx_Callback(hObject, eventdata, handles)
update_d_from_GUI;
update_pdf_graph;

%%

% Upper x value for probability calculation P( x_lower <= x < x_upper )
function EDITupperx_Callback(hObject, eventdata, handles)
update_d_from_GUI;
update_pdf_graph;

%%

% Edit box defining the number of samples taken from the skew-normal
% distribution, which forms the histogram and cumulative frequency graph plotted
function EDITN_Callback(hObject, eventdata, handles)
update_d_from_GUI;
generate_new_samples
update_histogram;

%%

% Push button which randomly chooses samples from the skew-normal
% distribution
function PUSHgensamples_Callback(hObject, eventdata, handles)
generate_new_samples;
update_histogram;

%%

% Use histogram edges defined in script HISTOGRAM_EDGES.m
function CHECKuseedges_Callback(hObject, eventdata, handles)
update_d_from_GUI;
update_histogram;

%%

% If the "use bin edges in HISTOGRAM_EDGES.m" box is NOT ticked, use
% d.num_bins linearly spaced bins between d.num_std either side of d.mean
function EDITnumbins_Callback(hObject, eventdata, handles)
update_d_from_GUI;
update_histogram;

%%

% Generate PNG files corresponding to the three graphs in the GUI
function PUSHpng_Callback(hObject, eventdata, handles)
global d

%Make a new directory
d.png_dir = ['normalgui graphs ',strrep( datestr(now), ':','-' )];
mkdir(d.png_dir);
update_pdf_graph(1);
update_histogram(1);

%%

% Increment mean and update graphs accordingly
function PUSHmeanplus_Callback(hObject, eventdata, handles)
global d h
d.x_mean = d.x_mean + d.delta_mean;
update_gui_edit_boxes_from_d;
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

% Decrement mean and update graphs accordingly
function PUSHmeanminus_Callback(hObject, eventdata, handles)
global d h
d.x_mean = d.x_mean - d.delta_mean;
update_gui_edit_boxes_from_d;
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

%Increment standard deviation and update graphs accordingly
function PUSHstdplus_Callback(hObject, eventdata, handles)
global d h
d.x_std = d.x_std + d.delta_std;
update_gui_edit_boxes_from_d;
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

% Decrement standard deviation and update graphs accordingly
function PUSHstdminus_Callback(hObject, eventdata, handles)
global d h
d.x_std = d.x_std - d.delta_std;
if d.x_std <=0
    d.x_std = 0.001;
end
update_gui_edit_boxes_from_d;
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

% Increment skew and update graphs accordingly
function PUSHskewplus_Callback(hObject, eventdata, handles)
global d h
d.x_skew = d.x_skew + d.delta_skew;
if d.x_skew >= 1
    d.x_skew = 0.99;
end
update_gui_edit_boxes_from_d;
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

% Decrement skew and update graphs accordingly
function PUSHskewminus_Callback(hObject, eventdata, handles)
global d h
d.x_skew = d.x_skew - d.delta_skew;
if d.x_skew <= -1
    d.x_skew = -0.99;
end
update_gui_edit_boxes_from_d;
update_d_from_GUI;
update_pdf_graph;
generate_new_samples
update_histogram;

%%

% Freeze graph scales to enable meaningful 'animation' of graphs with change of
% parameters
function TOGGLEfreezescale_Callback(hObject, eventdata, handles)
update_d_from_GUI;

%%

% Export an Excel file with samples, and also a frequency table based upon
% the parameters chosen for the histogram
function PUSHsamples2xls_Callback(hObject, eventdata, handles)
global d

%Write samples d.x_s to an Excel file
filename = ['skew-normal samples ',strrep( datestr(now), ':','-' )];
xlswrite( filename, d.x_s.' );

%Write frequency table Excel file
filename = ['skew-normal freq-table ',strrep( datestr(now), ':','-' )];
freq_table{1,1} = 'x range';
freq_table{1,2} = 'frequency';
for n=1:length(d.histogram_edges)-1
    freq_table{1+n,1} = [ num2str(d.histogram_edges(n)),' <= x < ',num2str(d.histogram_edges(n+1))];
    freq_table{1+n,2} = d.f(n);
end
freq_table{1+length(d.histogram_edges),1} = ['x = ',num2str(d.histogram_edges(end)) ];
freq_table{1+length(d.histogram_edges),2} = d.f(length(d.histogram_edges));
xlswrite( filename, freq_table );

%%

% Update GUI edit boxes from data array d
function update_gui_edit_boxes_from_d
global h d

%Default mean, standard deviation and skew parameters
set( h.EDITmean,'string',num2str( d.x_mean ) );
set( h.EDITstd,'string', num2str( d.x_std ) );
set( h.EDITskew,'string', num2str( d.x_skew) );

%Default values for probability calculation
set( h.EDITlowerx,'string', num2str( d.lower_x ) );
set( h.EDITupperx,'string', num2str( d.upper_x  ) );

%Number of data samples and histogram bins
set( h.EDITN,'string', num2str( d.num_samples ) );
set( h.EDITnumbins,'string', num2str( d.num_bins ) );

%Is graph scale frozen?
set( h.TOGGLEfreezescale, 'value', d.scale_frozen );

%Use histogram edges in script HISTOGRAM_EDGES.m
set( h.CHECKuseedges, 'value', d.use_histogram_edges );

%%

% Update data array d from GUI parameters
function update_d_from_GUI
global h d

%Get GUI data
h = guidata( h.FIGUREnormalgui );

%Default mean, standard deviation and skew parameters
d.x_mean = str2num( get( h.EDITmean,'string' ) );
d.x_std = str2num( get( h.EDITstd,'string' ) );
if d.x_std < 0
    d.x_std = abs( d.x_std );
end
set( h.EDITstd,'string',num2str( d.x_std ) );
d.x_skew = str2num( get( h.EDITskew,'string' ) );
if d.x_skew >= 1
    d.x_skew = 0.99;
elseif d.x_skew <= -1
    d.x_skew = -0.99;
end
set( h.EDITskew,'string',num2str( d.x_skew ) );

%Default values for probability calculation
d.lower_x = str2num( get( h.EDITlowerx,'string' ) );
d.upper_x = str2num( get( h.EDITupperx,'string' ) );

%Number of data samples and histogram bins
d.num_samples = str2num( get( h.EDITN,'string' ) );
d.num_bins =  str2num( get( h.EDITnumbins,'string' ) );

%Use histogram edges in script HISTOGRAM_EDGES.m
d.use_histogram_edges = get( h.CHECKuseedges,'value' );

%Is graph scale frozen?
d.scale_frozen = get( h.TOGGLEfreezescale,'value' );

%Define x limits
if d.scale_frozen == 0
    d.xmin = d.x_mean - d.num_std*d.x_std;
    d.xmax = d.x_mean + d.num_std*d.x_std;
    d.pdf_max = [];
end

%%

% Update skew-normal PDF graph and probability calculation
function update_pdf_graph(varargin)
global h d

%Point to AXESnormal in GUI, or if an extra argument is set, create a new
%figure and axes
if nargin==0
    fig = h.FIGUREnormalgui;
    ax = h.AXISnormal;
else
    fig = [];
    ax = [];
end

%Create PDF plot
[fig,ax,d.P,d.pdf_max] = plot_skewed_normal( d.x_mean, d.x_std, d.x_skew, d.lower_x,...
    d.upper_x, d.xmin, d.xmax,...
    d.pdf_max, d.fontsize, d.num_points, ax, fig );

%Update results box
set(h.TEXTprobability,'string',['P = ',num2str(d.P)]);

%Flush graphics buffer
drawnow;

%Print figure and close if function has no inputs
if nargin~=0
    print(gcf,[d.png_dir,'\probability'],'-dpng','-r300')
    close(gcf);
end

%%

% Generate a new set of samples and compute statistics
function generate_new_samples
global d

%Generate samples from skew-normal distribution
d.x = linspace( d.xmin, d.xmax, d.num_points );
[ d.x_s, pdf, cdf ] = skewed_normal( d.x_mean, d.x_std, d.x_skew, d.num_samples, d.x );

%%

% Update histogram and cumulative frequency graph
function update_histogram(varargin)
global d h

%Point to AXESnormal in GUI, or if an extra argument is set, create a new
%figure and axes
if nargin==0
    %Update axes in GUI and ignore un-normalized histogram
    fig_hist = [];
    ax_hist = [];
    fig_norm = h.FIGUREnormalgui;
    ax_norm = h.AXIShist;
    fig_cumf = h.FIGUREnormalgui;
    ax_cumf = h.AXIScumfreq;
    plot_hist = 0;
    full_title = 0;
else
    %Plot graphs in separate figures
    fig_hist =[];
    ax_hist = [];
    fig_norm =[];
    ax_norm = [];
    fig_cumf =[];
    ax_cumf =[];
    plot_hist = 1;
    full_title = 1;
end
plot_norm = 1;
plot_cumf = 1;

%Determine histogram bin edges
if d.use_histogram_edges == 1
    try
        run('HISTOGRAM_EDGES')
        min_bin_edge = min(histogram_edges);
        max_bin_edge = max(histogram_edges);
    catch
        histogram_edges = [];
        min_bin_edge = min(d.x_s);
        max_bin_edge = max(d.x_s);
    end
else
    histogram_edges = [];
    min_bin_edge = min(d.x_s);
    max_bin_edge = max(d.x_s);
end

%Determine statistics, histogram and cumulative frequency graphs
f = [];
[ fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf, d.histogram_edges, d.f  ] = ...
    plot_histogram_and_cum_freq( d.x_s, d.xmin, d.xmax, d.pdf_max, histogram_edges, f,...
    min_bin_edge, max_bin_edge, d.num_bins, d.num_points, d.fontsize, plot_hist, plot_norm,...
    plot_cumf, fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf, full_title );

%Flush graphics buffer
drawnow;

%Print figures and close if option set
if nargin~=0
    print(fig_hist,[d.png_dir,'\samples histogram'],'-dpng','-r300')
    close(fig_hist);
    print(fig_norm,[d.png_dir,'\samples normalized histogram'],'-dpng','-r300')
    close(fig_norm);
    print(fig_cumf,[d.png_dir,'\samples cum_freq'],'-dpng','-r300')
    close(fig_cumf);
end

%%

% Close
function CLOSE_Callback(hObject, eventdata, handles)
global h
delete( h.FIGUREnormalgui );
warning off
try
    rmpath('NormalStats');
end

%%

% Set default parameters
function d = normalgui_defaults

%Default mean, standard deviation and skew parameters
d.x_mean = 50;
d.x_std = 20;
d.x_skew = -0.7;

%+/- increments for mean, standard deviation and skew parameters
d.delta_mean = 0.1;
d.delta_std = 0.1;
d.delta_skew = 0.05;

%Set number of standard deviations either side of the mean which sets the
%x limits of graphs
d.num_std = 4;

%Define default x limits
d.xmin = d.x_mean - d.num_std*d.x_std;
d.xmax = d.x_mean + d.num_std*d.x_std;

%Default PDF grapg maximum
d.pdf_max = [];

%Default values for probability calculation
d.lower_x = 20;
d.upper_x = 60;

%Number of data samples, graph points and histogram bins
d.num_samples = 1000;
d.num_points = 500;
d.num_bins = 50;

%Is graph scale frozen?
d.scale_frozen = 1;

%Use histogram edges in script HISTOGRAM_EDGES.m
d.use_histogram_edges = 0;

%Font size for figures
d.fontsize = 14;

%End of code
