%stats_from_excel_freq_table
% Loads an excel file containing a frequency table of grouped data and performs a
% statistical analysis. Averages, standard deviation and skew are computed
% and histograms and cumulative frequency + box & whisker overlays are also
% constructed.
%
% LAST UPDATED by Andy French July 2014

function stats_from_excel_freq_table

% Default number of PDF overlay points and fontsize
num_points = 500;
fontsize = 14;

%

% Command line information
disp(' ')
disp(' STATISTICS FROM EXCEL FREQUENCY TABLE ' )
disp(' by Andy French. July 2014' )
disp(' ')

%Choose Excel file to load. This must contain a two columns. The first is a
%column of strings defining the variable x range, the second column is the
%frequency
[filename, pathname, filterindex] = uigetfile('*.xls', 'Choose an .xls file containing frequency table');
if filename ==0
    return
end

%Start clock
t0 = tic;

%Read Excel file and obtain array of sample x_s
disp(['Performing analysis of file: ',filename])
[num,txt,raw] = xlsread( [pathname,'\',filename] );
s = size(raw);

%Initialise histogram bins and frequencies
histogram_edges = [];
min_bin_edge = [];
max_bin_edge = [];
num_bins = [];
f = [];
for n=2:s(1)-1;
    %Extract text in first column up to ' <= ' and set this (following
    %convertion to a numeric value) as the histogram bin edge.
    r = raw{n,1};
    histogram_edges = [ histogram_edges, str2num( r( 1:strfind( raw{n,1},' <=' ) ) ) ];
    f = [f,raw{n,2}];
end

%Add final edge
histogram_edges = [ histogram_edges, str2num( r( strfind( raw{n,1},'x < ' )+4:end ) ) ];
f = [f,raw{ s(1),2}];

%Perform statistical analysis upon data samples
%Perform statistical analysis using stats_from_samples.m (called within
%plot_histogram_and_cum_freq.m) and plot graphs
pdf_max = [];
x_s = [];
plot_hist = 1;
plot_norm = 1;
plot_cumf = 1;
fig_hist=[];
ax_hist=[];
fig_norm=[];
ax_norm=[];
fig_cumf=[];
ax_cumf=[];
xmin = min(histogram_edges);
xmax = max(histogram_edges);
full_title = 1;
[ fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf, histogram_edges,f  ] = ...
    plot_histogram_and_cum_freq( x_s, xmin, xmax, pdf_max, histogram_edges,f,...
    min_bin_edge, max_bin_edge, num_bins, num_points, fontsize, plot_hist, plot_norm,...
    plot_cumf, fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf,...
    full_title );

%Create an output directory for graphs
output_dir = ['stats from xls freq table ',strrep( datestr(now), ':','-' )];
mkdir(output_dir);

%Print graphs as PNG files to directory and close associated figures
print(fig_hist,[output_dir,'\samples histogram'],'-dpng','-r300')
close(fig_hist);
print(fig_norm,[output_dir,'\samples normalized histogram'],'-dpng','-r300')
close(fig_norm);
print(fig_cumf,[output_dir,'\samples cum_freq'],'-dpng','-r300')
close(fig_cumf);

%Stop clock and display ending message
disp( ['Statistical analysis completed in ',num2str(toc(t0)),' seconds.'] )
disp(' ')

%End of code